#This program is used to do thge automatic crop of the hand region in the 4-steps sequence.
#Automatically finds the crop rect by analyzing all images
#
#  -----------		  
#  |  @	  |		-----
#  | @@@	 |		| @ |
#  |  @@	 |   =>   |@@@|
#  |		 |		| @@|
#  |		 |		-----
#  -----------
#
#Author: Andres J. Porfirio <www.andresjesse.com>, 15/05/2012


import sys,os,shutil
import cv #on linux you need to install the package: python-opencv
import numpy

DATABASE_FOLDER = 'IN/'
	
def calculateCropRectangle(img):
	threshold = 128

	def calculateProjV(img):
		proj = []
		for i in range(img.height):
			projValue = 0
			for j in range(img.width):
				if cv.Get2D(img,i,j)[0] > threshold:
					projValue = 1
					break

			proj.append(projValue)
		return proj
		
	def calculateProjH(img):
		proj = []
		for i in range(img.width):
			projValue = 0
			for j in range(img.height):
				if cv.Get2D(img,j,i)[0] > threshold:
					projValue = 1
					break

			proj.append(projValue)
		return proj	
		
	projv = calculateProjV(img)
	projh = calculateProjH(img)

	cropUp = projv.index(1)-1#offset (+1/-1) to avoid crop the hand
	projv.reverse()
	cropDown = len(projv)-projv.index(1)+1

	cropLeft = projh.index(1)-1
	projh.reverse()
	cropRight = len(projh)-projh.index(1)+1

	return cropLeft,cropRight,cropUp,cropDown

def create_cropped_folder(parent_dir):
	 if not os.path.exists(parent_dir+"cropped"):
		  os.makedirs(parent_dir+"cropped")

def crop_4step(folder):
	
	print "cropping sequence..."
	
	files = [f for f in os.listdir(folder)]#concatenate folder+file
	files = filter(lambda f: f.count(".jpg") > 0, files)#filter images
	
	#load images sequence and crop
	for i in range( len(files) ):
		img = cv.LoadImage(folder+files[i], cv.CV_LOAD_IMAGE_GRAYSCALE)
		
		#calculate crop rect
		cropLeft,cropRight,cropUp,cropDown = calculateCropRectangle(img)
		create_cropped_folder(folder)
	
		#crop images and store
		cv.SetImageROI(img,(cropLeft,cropUp,cropRight-cropLeft,cropDown-cropUp))
		cv.SaveImage(folder+"cropped/cropped_"+files[i],img)

def clean_old_crops():
	print 'Cleaning old crops...'
	for hcfg in os.listdir(DATABASE_FOLDER):
		if os.path.isdir(DATABASE_FOLDER+hcfg+'/cropped/'):
			#print 'Deleting:'+DATABASE_FOLDER+hcfg+'/cropped/'
			shutil.rmtree(DATABASE_FOLDER+hcfg+'/cropped/')

if __name__ == '__main__':
	clean_old_crops()

	count = 1

	for hcfg in os.listdir(DATABASE_FOLDER):
		folder = DATABASE_FOLDER+hcfg+'/'
		print 'Cropping:'+folder+' ('+str(count)+'/610)'
		crop_4step(folder)
		count += 1