#File: smooth.py
#Version: 1.1
#Author: Andres Jesse Porfirio (www.andresjesse.com)
#Date: 12/23/2012
#
#This script contains a method for smooth the kinect noise
#
#Update from 1.0: added a loop to process the entire database located in the "IN" 
#folder (result stored in "SOUT" folder)

import cv,sys,os

def smooth(in_file,out_file):
	blur_factor = 11
	threshold = 127

	#load the original image
	img = cv.LoadImage(in_file)
	#apply smooth
	cv.Smooth(img,img,cv.CV_GAUSSIAN,blur_factor,blur_factor)
	#apply threshold
	cv.Threshold(img, img, threshold, 255, cv.CV_THRESH_BINARY) 
	#save result
	cv.SaveImage(out_file,img)

#usage: process_database(IN,OUT), do not add '/' -> IN/
#database format:
#	IN/
#		C1_1_1/
#			out_1.jpg
#			out_3.jpg
#		C1_1_2/
#			out_34.jpg
#			out_38.jpg
def process_database(database_folder,result_folder):
	if not os.path.exists(result_folder):
		os.makedirs(result_folder)
	else:
		print 'Error:',result_folder,'already exists! aborting..'
		quit()

	for hcfg_dir in os.listdir(database_folder):
		current_dir = database_folder+'/'+hcfg_dir+'/'
		os.makedirs(result_folder+'/'+hcfg_dir)

		for img_file in os.listdir(current_dir):
			if not img_file[-4:] == '.jpg':
				print 'Skipping:',current_dir+img_file
				continue
			print 'Processing:',current_dir+img_file
			smooth(current_dir+img_file,result_folder+'/'+hcfg_dir+'/'+img_file)


if __name__ == '__main__':
	#uncomment the following block to use this script with commandline
	#dont forget to comment "process_database('IN', 'SOUT')" line
	#if len(sys.argv) < 3:
	#	print 'Usage: <in_img> <out_img>'
	#	print 'To change blur_factor and threshold edit the script..'
	#	quit()
	#smooth(sys.argv[1],sys.argv[2]) #version 1.0

	process_database('IN', 'SOUT') #version 1.1